﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace XNAGraffiti.ParticleSystems
{
    class FireParticleSystem : ParticleSystem
    {
        public FireParticleSystem(Game game, ContentManager content)
            : base(game, content)
        { }


        protected override void InitializeSettings(ParticleSettings settings)
        {
            settings.TextureName = "fire";

            settings.MaxParticles = 45000;

            settings.Duration = TimeSpan.FromSeconds(20);

            settings.DurationRandomness = 1;

            settings.MinHorizontalVelocity = 0;
            settings.MaxHorizontalVelocity = 0.15f;

            settings.MinVerticalVelocity = -0.05f;
            settings.MaxVerticalVelocity = 0.115f;

            // Set gravity upside down, so the flames will 'fall' upward.
            settings.Gravity = new Vector3(0, 0.5f, 0);

            settings.MinColor = new Color(10, 10, 10, 10);
            settings.MaxColor = new Color(255, 255, 255, 50);
            settings.MinRotateSpeed = -0.5f;
            settings.MaxRotateSpeed = 0.5f;
            settings.EmitterVelocitySensitivity = 14;

            settings.MinStartSize = 4;
            settings.MaxStartSize = 12;

            settings.MinEndSize = 12;
            settings.MaxEndSize = 15;

            // Use additive blending.
            settings.SourceBlend = Blend.SourceAlpha;
            settings.DestinationBlend = Blend.One;
        }
    }
}
